@@ -28,6 +28,7 @@ domain_nameRE=re.compile('^'+dot_atom+'$')
 email_addressRE=re.compile('^'+postfix_restricted_rfc2822_email_address+'$')
 valid_ipRE=re.compile('^([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$')
 valid_hostnameRE=re.compile('^(([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\\-]*[a-zA-Z0-9])\\.)*([A-Za-z]|[A-Za-z][A-Za-z0-9\\-]*[A-Za-z0-9])$')
+ftp_url_re=re.compile('ftp://(?:(?P<username>[\w\.\-\+]+)(?::(?P<password>[\w\.\-\+]+))?@)?(?P<host>[a-zA-Z0-9\.\-]+)(?::(?P<port>\d+))?(?:(?P<directory>/.*))?')
 
 # from FormEncode
 url_re = re.compile(r'''
@@ -436,8 +437,9 @@ class Destinations:
         
         self.destinations=[]
         st=raw
-        self.max_weekdivisor=1
+        self.full_cycle=7
         self.archiver=archiver
+        self.used_types=set()
         
         while st:
             if st[0]!='<':
@@ -460,7 +462,7 @@ class Destinations:
                 st=''
             
             typ, period=selector.split('=', 1)
-            
+
             try:
                 if typ.lower()=='none':
                     typ='none'
@@ -469,17 +471,21 @@ class Destinations:
             except KeyError:
                 raise DestinationSyntaxError, 'type "%s" unknown' % (typ, )
 
+            self.used_types.add(typ.lower())
+
             #print "SELECTOR<%s=%s>%s" % (typ,period,target )
             #print "CARRY=%s" % st
             period=cron.Cron(period)
             if period.weekdivisor!=None:
-                self.max_weekdivisor=max(self.max_weekdivisor, period.weekdivisor)
+                self.full_cycle=max(self.full_cycle, period.weekdivisor*7)
+            if period.months:
+                self.full_cycle=max(self.full_cycle, 63)
                 
             self.destinations.append((typ, period, target))
 
 
     def match(self, day, night_shift, variables={}):
-        #print '---------------------------------'
+        #print '---------------------------------', variables
         night_day=day
         if day.hour<12:
             night_day=night_day-timedelta(days=1)
@@ -497,12 +503,12 @@ class Destinations:
         globals['epoch']=str(int(epoch))
         globals['week']='0'
         globals['nweek']='0'
+        globals['nmonthweek']=night_day.day/7
         
         globals.update(self.archiver.variables)
         globals.update(variables)
-            
+
         for typ, period, target in self.destinations:
-            
             #print 'ASX typ, per, tgt', typ, period, target
             if period:
                 #print 'Month', backup_day.month-1, period.months
@@ -528,7 +534,7 @@ class Destinations:
                     globals['nweek']=week
 
             #print 'MATCH', typ, target
-            
+            rawtarget=target
             for exp in re.findall('\{([^}]*)\}', target):
                 try:
                     target=target.replace('${%s}' % (exp,), str(eval(exp, globals, {})))
@@ -545,6 +551,7 @@ class Destinations:
             
         return None, None
 
+
 # ---------------------------------------------------------------------------
 
 class MyMemoryHandler(logging.handlers.MemoryHandler):
@@ -567,13 +574,10 @@ def ErrorDecode(exception, encoding='utf-8'):
         return st.decode(encoding)
     except UnicodeDecodeError:
         return st.decode('ascii', 'replace')
-   
 
 # ---------------------------------------------------------------------------
 class Manager:
     
-    memory_log_capacity=2000
-
     def __init__(self, hostname, log):
         self.hostname=hostname
         self.log=log
@@ -585,19 +589,14 @@ class Manager:
         self.string_log=StringIO.StringIO()
         self.stream_log=logging.StreamHandler(self.string_log)
         self.stream_log.setFormatter(self.formater)
-        self.memory_log=MyMemoryHandler(self.memory_log_capacity, logging.INFO, self.stream_log)
-        self.log.addHandler(self.memory_log)
+        self.log.addHandler(self.stream_log)
 
     def stop_logging(self):
-        self.memory_log.flush()
         self.stream_log.flush()
-
-        self.log.removeHandler(self.memory_log)
-        
-        self.memory_log.close()
+        self.log.removeHandler(self.stream_log)
         self.stream_log.close()
         log_output=self.string_log.getvalue()
-        self.string_log.close()
+        # self.string_log.close() don't close it !
         return log_output
 
 # ---------------------------------------------------------------------------
@@ -626,6 +625,7 @@ class Archiver:
     
     # -----------------------------------------------------------------------
 
+
 # =====================================================================
 class UnixRegistry:
 
