@@ -4,27 +4,29 @@
 # mksbackup ghettovcb frontent
 #
 
-import os, string, subprocess, fnmatch, platform
-import posixpath
+import os, string, subprocess, fnmatch, platform, time
+import posixpath, ftplib
+import threading, random
 
 from datetime import datetime, timedelta
+import Queue
 
 import paramiko
 import scp
 
 
 from archiver import *
+import ftpd
 import cron
 
-GHETTO_VERSION='09/28/2010'
-#GHETTO_VERSION='02-13-2011_1'
+GHETTO_VERSION='2011_03_14_1'
 
 exit_code_re=re.compile('^\s*exit_code\s*=\s*([0-9]+)\s*$')
 
 # ---------------------------------------------------------------------------
 class MySCPClient(scp.SCPClient):
-    """MySCPClient.put will convert all '\r\n' into '\n'
-       MySCPClient.get will will download the remote dir at same level as local dir
+    """MySCPClient.put converts all '\r\n' into '\n'
+       MySCPClient.get downloads the remote dir at same level as local dir
     """ 
     
     # -----------------------------------------------------------------------
@@ -108,7 +110,7 @@ def RunCommand(t, command, log=None):
     exit_code=chan.recv_exit_status()
     chan.close()
     
-    # ESX(i) don't return any exit_code
+    # ESX(i) don't return any usable exit_code, use the one returned by  "echo exit_code=$?" 
     pos1=output.rfind('exit_code=')
     pos2=output.find('\n', pos1)
     exit_code=int(output[pos1+10:pos2])
@@ -141,7 +143,10 @@ class Ghettovcb(Archiver):
     initiate_backup_re=re.compile(r'.*info: Initiate backup for (?P<name>.+)')
     backup_duration_re=re.compile(r'.*info: Backup Duration:')
     config_vm_backup_volume_re=re.compile(r'.*info:\s*CONFIG\s*-\s*VM_BACKUP_VOLUME\s*=\s*(?P<vm_backup_volume>.+)')
-    
+    config_vm_backup_dir_naming_convention_re=re.compile(r'.*info:\s*CONFIG\s*-\s*VM_BACKUP_DIR_NAMING_CONVENTION\s*=\s*(?P<vm_backup_dir_naming_convention>.+)')
+    find_vmdk_re=re.compile(r'.*debug: findVMDK\(\) - Searching for VMDK: "(?P<vmdk>.+)" to backup')
+    ftp_url_re=ftp_url_re
+
     # -----------------------------------------------------------------------
     def __init__(self):
         Archiver.__init__(self)
@@ -169,12 +174,22 @@ class Ghettovcb(Archiver):
         self.scp_bin=None
         self.scp_args=None
         
+        self.upload_status=''
+        self.upload_status_code=None
+        self.upload_thread=None
+        self.upload_queue=Queue.Queue()
+        self.uploaded_filenames=dict()
+        self.ftp_local=None
+        self.ftp_local_default=None
+        self.ftp_server=None
+        self.ftp_thread=None  
+
     # -----------------------------------------------------------------------
     def read_vm_conf(self, filename, default={}):
         """load important variables from VM configuration backup script""" 
         values=default.copy()
         for line in open(filename, 'r'):
-            for var in ('VM_BACKUP_VOLUME', 'ENABLE_COMPRESSION', 'LAST_MODIFIED_DATE', 'VER', 'ENABLE_NON_PERSISTENT_NFS',):
+            for var in ('VM_BACKUP_VOLUME', 'ENABLE_COMPRESSION', 'LAST_MODIFIED_DATE', 'VER', 'VERSION', 'ENABLE_NON_PERSISTENT_NFS',):
                 if line.startswith('printUsage() {'):
                     break
                 if line.startswith('%s=' % var):
@@ -186,7 +201,7 @@ class Ghettovcb(Archiver):
     # -----------------------------------------------------------------------
     def load(self, job, manager):
         
-        log=manager.log
+        self.log=log=manager.log
         now=manager.now
         
         errors, warnings, extra={}, {}, ''
@@ -244,6 +259,15 @@ class Ghettovcb(Archiver):
 
         self.script=job.get('script', 'ghettoVCB.sh')
 
+        self.ftp_local=job.get('ftp_local', None)
+        if self.ftp_local:
+            if self.ftp_url_re.match(self.ftp_local):
+                password=self.ftp_url_re.match(self.ftp_local).group('password')
+                if password:
+                    self.register_password(password)
+            else:
+                errors['ftp_local']='invalid url: ftp://[username[:password]@]host[:port]/rootdir'
+
         if not os.path.isdir(self.local):
             errors['local']='not a directory'
         else:
@@ -280,13 +304,12 @@ class Ghettovcb(Archiver):
             if self.ghettovcb:
                 self.default_var=self.read_vm_conf(os.path.join(self.local, self.ghettovcb))
                 # check ghettoVCB version
-                self.ghettovcb_version=self.default_var.get('LAST_MODIFIED_DATE','')+self.default_var.get('VER','')
+                self.ghettovcb_version=self.default_var.get('LAST_MODIFIED_DATE','')+'_'+(self.default_var.get('VER','') or self.default_var.get('VERSION', ''))
                 self.ghettovcb_wait_nfs=self.ghetto_version_format1.match(self.default_var.get('LAST_MODIFIED_DATE',''))!=None
-                log.error('ghettovcb_version %r %r', self.ghettovcb_version, self.ghettovcb_wait_nfs)
                 if self.ghettovcb_wait_nfs:
                     log.debug('this version of ghettovcb wait before to unmount the NFS share')
                 
-                if self.default_var.get('LAST_MODIFIED_DATE','')!=GHETTO_VERSION:
+                if self.ghettovcb_version!=GHETTO_VERSION:
                     warnings['local']='This version of ghettoVCB.sh (%s) script has not been tested with MKSBackup, be warned ! Check the download page for more.' % (self.ghettovcb_version, )
                 
             # read and extract global_conf
@@ -313,10 +336,20 @@ class Ghettovcb(Archiver):
                 if not os.path.isfile(self.scp_bin):
                     errors['scp_bin']='file not found'
 
-        if not errors and self.destination:
-            self.type, self.target=self.destination.match(now, self.night_shift, variables=dict(vm=self.vm_list[0]))
-            if self.type in ('move', 'copy') and not os.path.isdir(os.path.dirname(self.target)):
-                errors['destination']='directory not found: %s' % (os.path.dirname(self.target), )
+        if not errors and self.destination and self.vm_list:
+            # very limited check of target directory
+            for vm in self.vm_list:
+                type, target=self.destination.match(now, self.night_shift, variables=dict(vm=vm))
+                # I cannot check more than dirname because when COMPRESSED is 1
+                # target is a filename and not a directory 
+                if type in ('move', 'copy') and not target.startswith('ftp://'):
+                    dirname=os.path.dirname(target)
+                    try:
+                        os.makedirs(dirname)
+                    except:
+                        pass
+                    if not os.path.isdir(dirname):
+                        errors['destination']='directory not found: %s' % (dirname, )
                     
         if not errors:
             # upload and setup files on the VMWARE server
@@ -329,10 +362,11 @@ class Ghettovcb(Archiver):
 
             self.temp_target=posixpath.join(self.remote_temp, os.path.basename(self.local))
             self.ghettovcb_bin=posixpath.join(self.temp_target, self.ghettovcb)
+            
             self.include_vm_file=posixpath.join(self.temp_target, 'include_vm')
             self.exclude_vm_file=posixpath.join(self.temp_target, 'exclude_vm')
             
-            _exit_code, _output=RunCommand(t, 'chmod +x "%s"' % self.ghettovcb_bin, log)
+            _exit_code, _output=RunCommand(t, 'chmod +x "%s"' % (self.ghettovcb_bin, ), log)
 
             self.cmd_line='cd "%s" ; ./"%s" -d debug -c "%s"' % (self.temp_target, self.ghettovcb, self.temp_target)
             
@@ -369,17 +403,250 @@ class Ghettovcb(Archiver):
                 
         return errors, warnings, extra
 
+    # -----------------------------------------------------------------------
+    def upload_queue_add(self, protocol, con, vm, source, destination):
+        self.upload_queue.put((protocol, con, vm, source, destination))
+        if not self.upload_thread:
+            self.upload_thread=threading.Thread(target=self.uploader_thread, name='uploader')
+            self.upload_thread.daemon=True
+            self.upload_thread.start()
+        
+    # -----------------------------------------------------------------------
+    def uploader_thread(self):
+        current_vm=None
+        self.upload_status=''
+        if self.upload_status_code==None:
+            self.upload_status_code='OK'
+        while True:
+            protocol, con, vm, source, destination=self.upload_queue.get(True)
+            if vm!=current_vm:
+                if current_vm!=None:
+                    # just finished  current_vm
+                    if status!='OK':
+                        self.upload_status_code='ERR'
+                    self.upload_status+='upload_status-%s=%s\r\n' % (current_vm, status)
+                    self.upload_status+='upload_end-%s=%s\r\n' % (current_vm, time.ctime(end))
+                    self.upload_status+='upload_end_epoch-%s=%d\r\n' % (current_vm, end)
+
+                start=time.time()
+                current_vm=vm
+                if current_vm!=None:
+                    # start a new vm
+                    start=time.time()
+                    status='OK'
+                    self.upload_status+='upload_start-%s=%s\r\n' % (current_vm, time.ctime(start))
+                    self.upload_status+='upload_start_epoch-%s=%d\r\n' % (current_vm, start)
+                    self.upload_status+='upload_src-%s=%s\r\n' % (current_vm, source)
+                    self.upload_status+='upload_dst-%s=%s\r\n' % (current_vm, destination)
+
+            if protocol=='done':
+                break
+                                           
+            if protocol=='ftp':
+                ftp_line='ftpput -P %s -u "%s" -p "%s" %s %s %s' % (con['port'], con['username'], con['password'], con['host'], destination, source)
+                self.log.info('start upload of "%s" to "%s"', source, destination)
+                cmd_line=self.hide_password(ftp_line)
+                self.log.debug('ftp command: %s', cmd_line)
+                exit_code, output=RunCommand(self.t, ftp_line, log=None)
+                if exit_code==0:
+                    self.log.debug('file uploaded: %s', destination)
+                else:
+                    status='ERR'
+                    self.log.error('upload failed %s (%s)', destination, output)
+            elif protocol=='ssh':
+                self.log.info('start download from "%s" to "%s"', source, destination)
+
+                if not self.scp_bin:
+                    # use paramiko
+                    try:
+                        scpclient=MySCPClient(self.t, buff_size=65536)
+                        self.log.debug('scp %s %s', source, destination)
+                        scpclient.get(source, os.path.dirname(destination))
+                    except scp.SCPException:
+                        self.log.error('download failed: %s (%s)', destination, output)
+                        status='ERR'
+                else:
+                    # use local SCP
+                    scp_args=self.scp_args[:]
+                    scp_args+=[ '%s@%s:%s' % (self.login, self.host, source), destination]
+                    scp_cmd=self.hide_password(' '.join(scp_args))
+                    self.log.debug('scp_cmd=%s', scp_cmd)
+                    if sys.platform in ('win32', ):
+                        scp_args=param_encode(scp_args, self.manager.default_encoding)
+                    scp_process=subprocess.Popen(scp_args, executable=self.scp_bin, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+                    scp_out, scp_err=scp_process.communicate()
+                    if scp_out:
+                        self.log.debug('copying %s: %s', source, scp_out)
+                    if scp_err:
+                        self.log.debug('copying %s: %s', source, scp_err)
+
+                    if scp_process.returncode!=0:
+                        self.log.error('download failed: %s (%s)', destination, scp_out+scp_err)
+                        status='ERR'
+
+            end=time.time()
+        self.upload_status='upload_status=%s\r\n%s' % (self.upload_status_code, self.upload_status)
+
+    # -----------------------------------------------------------------------
+    def start_ftp_server(self):
+        if self.ftp_server:
+            return True
+
+        rndpass=reduce(lambda x, y:x+random.choice('abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'), range(8), '')
+        self.register_password(rndpass)
+
+        if self.ftp_local:  
+            ftp=self.ftp_url_re.match(self.ftp_local).groupdict()
+        else:
+            ftp=dict(host='localhost')
+
+        for k, v in dict(username=rndpass, password=rndpass, directory='/', port='0').iteritems():
+            if ftp.get(k)==None:
+                ftp[k]=v
+        
+        if ftp['host']=='localhost':
+            ftp['host']=socket.gethostbyname(socket.gethostname())
+            host=''        
+        else:
+            host=socket.gethostbyname(ftp['host'])
+
+        if sys.platform in ('win32', ):
+            # convert posix directory into windows path 
+            # /c/backup => C:\backup 
+            root=ftp['directory']
+            if re.match('^/[a-zA-Z]/', root):
+                root=root[1]+':'+root[2:].replace('/', '\\')
+            else:
+                root=''
+            ftp['directory']=root
+        
+        
+        try:
+            if ftp['directory']:
+                os.makedirs(ftp['directory'])
+        except:
+            pass
+        
+        if not ftp['directory'] or not os.path.isdir(ftp['directory']):
+            self.log.error('Cannot start built-in ftp server, wrong root directory: "%s"', ftp['directory'])            
+            return False
+
+        self.ftp_server=ftpd.FTPserver(addr=(host, int(ftp['port'])), root=ftp['directory'], user=ftp['username'], password=ftp['password']) # allowed=self.host
+        self.ftp_thread=threading.Thread(target=self.ftp_server.serve_forever, name='ftpserver', )
+        self.ftp_thread.daemon=True
+        self.ftp_thread.start()
+        
+        ftp['port']=self.ftp_server.socket.getsockname()[1]
+        self.ftp_local_default=ftp
+
+        return True
+         
+    # -----------------------------------------------------------------------
+    def upload_vmdk(self, vm, root_backup_dir, vmdk, now):
+        type, target=self.destination.match(now, self.night_shift, variables=dict(vm=vm))
+        # print "ASX upload_vmdk vm=%s type=%s target=%s root_backup_dir=%s vmdk=%s" %(vm, type, target, root_backup_dir, vmdk)
+        if target and type in ('move', 'copy'):
+            ftp_match=self.ftp_url_re.match(target)
+
+            if ftp_match:
+                ftp=ftp_match.groupdict()
+                if ftp['directory']==None:
+                    ftp['directory']=''
+                    
+                if ftp['host']=='localhost':
+                    if not self.ftp_server:
+                        try:
+                            res=self.start_ftp_server()
+                        except Exception, e:
+                            self.log.exception('starting ftp server')
+                            return
+                        else:
+                            if res:
+                                self.log.info('ftpserver listening on %s:%d', self.ftp_server.socket.getsockname()[0], self.ftp_server.socket.getsockname()[1])
+                            else:
+                                return
+                    ftp['host']=self.ftp_local_default['host']
+                    for k in ('port', 'username', 'password'):
+                        if ftp.get(k)==None:
+                            ftp[k]=self.ftp_local_default[k]
+                
+                else:
+                    # try to create directories on the remote ftp server from 
+                    # this host. If it fail the directory must already exist.   
+                    try:
+                        ftpcli=ftplib.FTP(ftp['host'], ftp['username'], ftp['password'], timeout=10)
+                        dir=ftp['directory']
+                        dirs=[]
+                        while dir and dir!='/':
+                            dirs.insert(0, dir)
+                            dir=posixpath.dirname(dir)
+                            print dir
+
+                        for dir in dirs:
+                            try:
+                                ftpcli.mkd(dir)
+                            except:
+                                pass
+                        try:
+                            ftpcli.quit()
+                        except:
+                            pass
+                        ftpcli.close()
+                    except:
+                        pass
+                            
+                                
+                target=ftp['directory']
+                protocol, con='ftp', ftp
+                
+            else:
+                # create the target dir, if don't exist
+                try:
+                    if not os.path.isdir(target):
+                        os.makedirs(target)
+                except Exception, e:
+                    self.log.error('creating directory %s: %s', target, e)
+                    return
+
+                protocol, con='ssh', self.t
+
+            # search for files older than vmdk
+            if vmdk:
+                # file older then vmdk
+                find_cmd='find "%s" -type f \! -newer "%s"' % (root_backup_dir, posixpath.join(root_backup_dir, vmdk))
+            else:
+                # all files in the directory
+                find_cmd='find "%s" -type f' % (root_backup_dir, )
+                
+            exit_code, output=RunCommand(self.t, find_cmd, log=None)
+            # print 'ASX\n', output
+            if exit_code:
+                self.log.error('cannot list %s: %r',root_backup_dir, output)
+                self.upload_status_code='ERR'                
+            else:
+                for filename in output.split('\n'):
+                    filename=filename.strip()
+                    if filename and filename not in self.uploaded_filenames:
+                        self.uploaded_filenames[filename]=None
+                        relfilename=filename[len(root_backup_dir):]
+                        relfilename=relfilename.lstrip('/')
+                        if protocol=='ftp':
+                            destination=posixpath.join(target, relfilename)
+                        else:
+                            destination=os.path.join(target, relfilename)
+                        self.upload_queue_add(protocol, con, vm, filename, destination)
+
     # -----------------------------------------------------------------------
     def run(self, command, job, manager):
 
-        any_error=False
-
-        log=manager.log
+        self.log=log=manager.log
+        self.manager=manager
+        
         now=manager.now
-        log.info('ghettoVCB.sh version is: %s' % self.ghettovcb_version)
+        log.info('ghettoVCB.sh version is: %s  nfs_wait=%r', self.ghettovcb_version, self.ghettovcb_wait_nfs)
         start=int(time.time())
 
-        t=paramiko.Transport((self.host, self.port))
+        self.t=t=paramiko.Transport((self.host, self.port))
         # t.get_security_options()._set_ciphers(['blowfish-cbc', ]) # don't speed up anything and don't work with 3.5 
         t.connect(username=self.login, password=self.password)
         # t.set_keepalive(300) # make the backup fail
@@ -388,36 +655,33 @@ class Ghettovcb(Archiver):
         log.info('start: %s', self.cmd_line)
         chan.exec_command(self.cmd_line)
 
-        # search in the output for the name of backed up vm
-        # should be == to self.vm_list when provided
         attachments=[]
         
-        vm_list=[] # that need to be scp somewhere 
-
         output=''
         state='waiting_for_start'
+        current_vmdk=None
         for line in socket_readlines(chan):
             # I'm reading the socket in this loop,
-            # I CANNOT DO SOMETHING TIME CONSUMING like scp 
+            # I CANNOT DO SOMETHING TIME CONSUMING or do it in background 
             output+=line+'\n'
             log.debug(line)
             
-            current_error=False
-            
             match=self.config_vm_backup_volume_re.match(line)
             if match:
                 vm_backup_volume=match.group('vm_backup_volume')
+                continue
+
+            match=self.config_vm_backup_dir_naming_convention_re.match(line)
+            if match:
+                vm_backup_dir_naming_convention=match.group('vm_backup_dir_naming_convention')
+                continue
             
             match=self.initiate_backup_re.match(line)
             if match:
+                # start a new VM
                 state='started'
                 current_vm=match.group('name')
-                continue
-            
-            match=self.backup_duration_re.match(line)
-            if match:
-                """the backup just finished"""
-            
+
                 # read vm config file if it exist
                 filename=os.path.join(self.local, current_vm)
                 if os.path.isfile(filename):
@@ -425,17 +689,39 @@ class Ghettovcb(Archiver):
                 else:
                     vm_var=self.default_var
                     
+                # use vm_backup_volume instead of vm_var['VM_BACKUP_VOLUME']
+                # because the first one as been recalculated by ghettoVCB to take care of 
+                # ENABLE_NON_PERSISTENT_NFS
+                target=posixpath.join(vm_backup_volume, current_vm)
+                compress=vm_var['ENABLE_COMPRESSION']=='1'
+
+                continue
+
+            match=self.find_vmdk_re.match(line)
+            if match:
+                # start a new disk
+                if current_vmdk and vm_var['ENABLE_NON_PERSISTENT_NFS']!='1' and self.destination:
+                    # if needed, now we can upload the previous disk
+                    _res=self.upload_vmdk(current_vm, root_backup_dir, current_vmdk, now)
+                     
+                # store current vmdk
+                current_vmdk=match.group('vmdk')
+                root_backup_dir=posixpath.join(vm_backup_volume, current_vm, current_vm+'-'+vm_backup_dir_naming_convention)
+                continue
+
+            match=self.backup_duration_re.match(line)
+            if match:
+                """the backup just finished"""
+                if current_vmdk and vm_var['ENABLE_NON_PERSISTENT_NFS']!='1' and self.destination:
+                    # if needed, now we can upload the previous disk
+                    _res=self.upload_vmdk(current_vm, root_backup_dir, current_vmdk, now)
+            
                 # search the target and make a directory listing 
                 
-                dir_lst, up_lst, scp_process, target=None, None, None, None
+                dir_lst, up_lst, scp_process=None, None, None
                 if vm_var['ENABLE_NON_PERSISTENT_NFS']=='1' and not self.ghettovcb_wait_nfs:
                     log.warning('cannot retrieve data from non persistent NFS share on old version of ghettoVCB.sh')
                 else:
-                    # use vm_backup_volume instead of vm_var['VM_BACKUP_VOLUME']
-                    # because the first one as been recalculated by ghettoVCB to take care of 
-                    # ENABLE_NON_PERSISTENT_NFS
-                    target=posixpath.join(vm_backup_volume, current_vm)
-                    compress=vm_var['ENABLE_COMPRESSION']=='1'
                     if compress:
                         _exit_code, dir_lst=RunCommand(t, 'ls -l "%s"' % target, log)
                         dir_lst='== directory '+target+'\n'+dir_lst
@@ -462,21 +748,11 @@ class Ghettovcb(Archiver):
                     if up_lst:
                         attachments.append(('dirup-%s.txt' % (current_vm,), None, up_lst, 'text', 'plain', 'utf-8'))
             
-            
-                    if vm_var['ENABLE_NON_PERSISTENT_NFS']!='1' and self.destination:
-                        # each VM could have a different ENABLE_NON_PERSISTENT_NFS
-                        # this is not true but ....
-                        # if destination, move or copy backup
-                        vm_list.append((current_vm, compress, target))
-
-
-            any_error=any_error or current_error
         
-            
         ghettovcb_exit_code=chan.recv_exit_status()
         chan.close()
         # log.debug('ghettovcb return unreliable ssh exit code: %d', ghettovcb_exit_code)
-        end=int(time.time())
+        end_ghettovcb=int(time.time())
         
         # ESX(i) (dropbear) returns an unreliable exit_code   
         pos1=output.rfind('exit_code=')
@@ -484,94 +760,6 @@ class Ghettovcb(Archiver):
         ghettovcb_exit_code=int(output[pos1+10:pos2])
         output=output[:pos1]
 
-        status=''
-        for current_vm, compress, target in vm_list:
-            # if destination is set, and type is move or copy backup
-            # handle ssh scp and file delete
-            current_error=False
-            if self.destination:
-                self.type, self.target=self.destination.match(now, self.night_shift, variables=dict(vm=current_vm))
-                if self.type in ('move', 'copy'):
-                    if not compress:
-                        # create the target dir, if don't exist
-                        try:
-                            if not os.path.isdir(self.target):
-                                os.makedirs(self.target)
-                        except Exception, e:
-                            log.error('creating directory %s: %s', self.target, e)
-                            current_error=True
-
-                    if not current_error:
-                        start_copy=int(time.time())
-                        status+='remote_target-%s=%s\r\n' % (current_vm, target)
-                        status+='target-%s=%s\r\n' % (current_vm, self.target)
-                        if not self.scp_bin:
-                            # use paramiko
-                            try:
-                                scpclient=MySCPClient(t, buff_size=65536)
-                                log.info('scp -r %s %s', target, self.target)
-                                scpclient.get(target, self.target, recursive=True)
-                            except SCPException:
-                                returncode=1
-                                current_error=True
-                            else:
-                                returncode=0
-                            status+='scp_exit_code-%s=%d\r\n' % (current_vm, returncode)
-                        else:
-                            # use local SCP
-                            scp_args=self.scp_args[:]
-                            if not compress:
-                                if sys.platform in ('win32', ):
-                                    scp_args.append('-unsafe')
-                                #scp_args.append('-r')
-                                wildcard='/*'
-                            else:
-                                wildcard=''
-                                
-                            scp_args+=[ '%s@%s:%s%s' % (self.login, self.host, target, wildcard), self.target]
-                            scp_cmd=self.hide_password(' '.join(scp_args))
-                            log.info('scp_cmd=%s', scp_cmd)
-                            if sys.platform in ('win32', ):
-                                scp_args=param_encode(scp_args, manager.default_encoding)
-                            scp_process=subprocess.Popen(scp_args, executable=self.scp_bin, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-                            scp_out, scp_err=scp_process.communicate()
-                            if scp_out:
-                                attachments.append(('scp_out-%s.txt' % (current_vm,), None, scp_out, 'text', 'plain', manager.console_encoding))
-                            if scp_err:
-                                attachments.append(('scp_err-%s.txt' % (current_vm,), None, scp_err, 'text', 'plain', manager.console_encoding))
-                            status+='scp_cmd-%s=%s\r\n' % (current_vm, scp_cmd)
-                            status+='scp_exit_code-%s=%d\r\n' % (current_vm, scp_process.returncode)
-                            
-                            if scp_process.returncode!=0:
-                                current_error=True
-                            
-                        log.info('scp end')
-                        
-                        if self.type=='move':
-                            # delete the source
-                            log.info('delete %s', target)
-                            _exit_code, _rm_out=RunCommand(t, 'rm -r "%s"' % target, log)
-    
-                        try:
-                            total_free_bytes=free_space(self.target, log)
-                        except Exception, e:
-                            log.error('checking target directory: %s', e)
-                            status+='target_free_space-%s=%s\r\n' % (current_vm, e)
-                        else:
-                            status+='target_free_space-%s=%d\r\n' % (current_vm, total_free_bytes)
-            
-                        dir_out, _total_size=list_dir(self.target, log)        
-                        attachments.append(('target_dir-%s.txt' % (current_vm,), None, dir_out.encode('utf-8'), 'text', 'plain', 'utf-8'))
-
-                        end_copy=int(time.time())
-                        status+='scp_start-%s=%s\r\n' % (current_vm, time.ctime(start_copy))
-                        status+='scp_end-%s=%s\r\n' % (current_vm, time.ctime(end_copy))
-                        status+='scp_start_epoch-%s=%d\r\n' % (current_vm, start_copy)
-                        status+='scp_end_epoch-%s=%d\r\n' % (current_vm, end_copy)
-
-            any_error=any_error or current_error
-
-
         attachments.insert(0, ('output.txt', None, output, 'text', 'plain', 'utf-8') )
         
         # make the disk space usage
@@ -581,34 +769,51 @@ class Ghettovcb(Archiver):
         _exit_code, ls_volumes=RunCommand(t, 'ls -l /vmfs/volumes', log)
         ls_volumes='== directory /vmfs/volumes\n'+ls_volumes
 
+        # Nothing more to upload    
+        self.upload_queue_add('done', None, None, None, None)
+        # wait for end of upload /download
+        if not self.upload_queue.empty():
+            log.info('waiting end of upload...')
+            while not self.upload_queue.empty():
+                time.sleep(5)
+
+        end=int(time.time())
+
         t.close()
 
-        if ghettovcb_exit_code==0 and not any_error:
+        if ghettovcb_exit_code==0:
+            ghettoVCB_status='OK'
             backup_status='OK'
         else:
+            ghettoVCB_status='ERR'
             backup_status='ERR'
 
-        status_pre=u''
-        status_pre+='name=%s\r\n' % job['name']
-        status_pre+='program=%s\r\n' % self.name
-        status_pre+='version=%s\r\n' % self.__version__
-        status_pre+='status=%s\r\n' % backup_status
-        status_pre+='hostname=%s\r\n' % manager.hostname
-        
-        status_pre+='exit_code=%d\r\n' % ghettovcb_exit_code
-        status_pre+='start=%s\r\n' % time.ctime(start)
-        status_pre+='end=%s\r\n' % time.ctime(end)
-        status_pre+='start_epoch=%d\r\n' % start
-        status_pre+='end_epoch=%d\r\n' % end
+        if self.upload_status_code and self.upload_status_code!='OK':
+            backup_status='ERR'
 
-        status_pre+='type=%s\r\n' % self.type
-        status=status_pre+status
+        status=u''
+        status+='name=%s\r\n' % job['name']
+        status+='program=%s\r\n' % self.name
+        status+='version=%s\r\n' % self.__version__
+        status+='status=%s\r\n' % backup_status
+        status+='hostname=%s\r\n' % manager.hostname
         
+        status+='exit_code=%d\r\n' % ghettovcb_exit_code
+        status+='start=%s\r\n' % time.ctime(start)
+        status+='end_ghettovcb=%s\r\n' % time.ctime(end_ghettovcb)
+        status+='end=%s\r\n' % time.ctime(end)
+        status+='start_epoch=%d\r\n' % start
+        status+='end_epoch=%d\r\n' % end
+        status+='end_ghettovcb_epoch=%d\r\n' % end_ghettovcb
+
+        status+='type=%s\r\n' % self.type
+        status+='ghettovcb_status=%s\r\n' % ghettoVCB_status
+        status+=self.upload_status
+
         if df:
             attachments.insert(0, ('df.txt', None, df, 'text', 'plain', 'utf-8'),)
 
         if ls_volumes:
             attachments.insert(0, ('ls_volumes.txt', None, ls_volumes, 'text', 'plain', 'utf-8'),)
-
         
         return backup_status, status, attachments
