@@ -31,7 +31,7 @@ import ConfigParser
 
 from datetime import datetime, timedelta
 
-from archiver import boolean, check_mail_config, Destinations, sendmail, Manager, send_mail_report
+from archiver import boolean, check_mail_config, Destinations, sendmail, Manager, send_mail_report, ftp_url_re
 
 import cron
 
@@ -44,6 +44,24 @@ from version import __version__
 
 last_version_url='http://www.magikmon.com/download/mksbackup/last2.txt'
 
+def repl_ftp_password(m):
+    """used by re.sub to replace ftp password by ***********"""
+    username, password, host, port, directory=m.group('username', 'password', 'host', 'port', 'directory')
+    st='ftp://'
+    if username:
+        host='@'+host
+        st+=username
+    if password:
+        st+=':********'
+        
+    st+=host
+    if port:
+        st+=':'+port
+    if directory:
+        st+=directory
+    return st
+        
+        
 def gen_archiver(name):
     name=name.lower()
     if name=='ntbackup':
@@ -198,6 +216,7 @@ class MyFormatter(logging.Formatter):
 # Parse the command line
 #
 def main():
+    
     parser=parser=OptionParser(version='%%prog %s' % __version__ )
     parser.set_usage('%prog [options] command [job..]\n\n'
                       '\tcommand in "backup", "check", "checkmail"')
@@ -209,15 +228,18 @@ def main():
     parser.add_option("-l", "--logfile", dest="logfile", default='mksbackup.log', help="log to this file", metavar="logfile")
     parser.add_option("-e", "--encoding", dest="encoding", default='auto', help="configuration file encoding", metavar="encoding")
     
-    installable=sys.platform in ('win32', ) # TODO  and sys.argv[0][:4].lower()=='.exe'
-    if installable:
-        parser.add_option("-i", "--install", dest="install", action="store_true", help="install and pre-setup a task in scheduler")
+    parser.add_option("-i", "--install", dest="install", action="store_true", help="install and pre-setup a task in scheduler")
     
     cmd_options, cmd_args=parser.parse_args(sys.argv)
     
-    if installable and (cmd_options.install or len(sys.argv)==1):
-        from windows import Install
-        sys.exit(Install())
+    if cmd_options.install or len(sys.argv)==1:
+        import install
+        try:
+            res=install.Install()
+        except Exception, e:
+            print 'Unexpected Error: %s' % (e, )
+            raise
+        sys.exit(res)
     
     root_handler=logging.handlers.RotatingFileHandler(cmd_options.logfile, 'a', 1024**2, 2)
     root_handler.setFormatter(logging.Formatter('%(asctime)s %(levelname)-3.3s %(message)s', '%Y-%m-%d %H:%M:%S'))
@@ -232,6 +254,8 @@ def main():
         # improve performance a little bit
         logging.disable(logging.DEBUG)
     
+    logging.getLogger("paramiko").setLevel(logging.WARNING)
+        
     if cmd_options.verbose:
         console=logging.StreamHandler()
         encoding=sys.stderr.encoding
@@ -323,14 +347,14 @@ def main():
         try:
             config_file=codecs.open(cmd_options.config, 'r', cmd_options.encoding)        
         except LookupError, e:
-            log.error('error reading configuration file %s: %s', cmd_options.config, ErrorDecoding(e, default_encoding))
+            log.error('error reading configuration file %s: %s', cmd_options.config, ErrorDecode(e, default_encoding))
             sys.exit(1)
     
     config=ConfigParser.RawConfigParser(config_default)
     try:
         config.readfp(config_file)
     except (IOError, UnicodeDecodeError), e:
-        log.error('error reading configuration file %s: %s', cmd_options.config, ErrorDecoding(e, default_encoding))
+        log.error('error reading configuration file %s: %s', cmd_options.config, ErrorDecode(e, default_encoding))
         sys.exit(1)
     
     err, failed_job=0, job_list[:]
@@ -375,7 +399,9 @@ def main():
     # remove password from .INI file 
     sanitized_config_file_content=codecs.open(cmd_options.config, 'r', cmd_options.encoding).read()
     for password_key in ('password', 'smtp_password',):
-        sanitized_config_file_content=re.sub('(?m)^#?(?P<key>\s*%s\s*)=(?P<value>.*$)' % (password_key,), '\g<key>=********', sanitized_config_file_content)
+        sanitized_config_file_content=re.sub('(?m)^#?(?P<key>\s*%s\s*)=(?P<value>.*)$' % (password_key,), '\g<key>=********', sanitized_config_file_content)
+
+    sanitized_config_file_content=ftp_url_re.sub(repl_ftp_password, sanitized_config_file_content)
     
     manager=Manager(platform.node(), log)
     manager.default_encoding=default_encoding
@@ -383,6 +409,7 @@ def main():
     manager.formater=MyFormatter('%(asctime)s %(levelname)-3.3s %(message)s', '%Y-%m-%d %H:%M:%S', encoding='utf-8')
 
     manager.sanitized_config_file_content=sanitized_config_file_content
+    
     manager.attachments=[ ('config.ini', None, manager.sanitized_config_file_content.encode(cmd_options.encoding), 'text', 'plain', cmd_options.encoding), ] 
     
     for job_name in job_list:
@@ -417,7 +444,7 @@ def main():
             try:
                 log.exception('loading job %s by program %s', job_name, program)
             except UnicodeDecodeError:
-                log.error('loading job %s by program %s: %s', job_name, program, ErrorDecoding(e, default_encoding))
+                log.error('loading job %s by program %s: %s', job_name, program, ErrorDecode(e, default_encoding))
                 
             if job['mail'] and not mail_config_errors:
                 subject='MKSBACKUP %s ERR %s ' % (command_display, job['name'], )
@@ -467,7 +494,7 @@ def main():
             if destination and isinstance(destination, Destinations): 
                 msg_body+='Destinations day by day:\n\n'
                 log.info('Destinations day by day:')
-                for i in range(destination.max_weekdivisor*7):
+                for i in range(destination.full_cycle):
                     today=now+timedelta(days=i)
                     typ, target=destination.match(today, night_shift=arch.night_shift)
                     if typ=='none':
@@ -505,7 +532,7 @@ def main():
             try:
                 log.exception('running command=%s job=%s archiver=%s', command, job_name, program)
             except UnicodeDecodeError:
-                log.error('running command=%s job=%s archiver=%s : %s', command, job_name, program, ErrorDecoding(e, default_encoding))
+                log.error('running command=%s job=%s archiver=%s : %s', command, job_name, program, ErrorDecode(e, default_encoding))
 
             if job['mail']:
                 subject='MKSBACKUP BACKUP ERR %s ' % (job['name'], )
@@ -523,6 +550,7 @@ def main():
 
         
     log.info('final exit code=%d', exit_code)
+
     return exit_code
     
 if __name__ == "__main__":
